"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const util_1 = __importDefault(require("util"));
/**
 * Rule Input Provider Amqp
 *
 * This class wires up the AmqpCacoon to a rule input provider.
 *
 * Usage:
 * 1. Instantiate the class and pass in the instantiated amqpCacoon and logger
 * 2. call registerHandler to register the input callback
 * 3. When input comes in, the handler registered in step 2 will be called
 *
 **/
class CoreInputAmqp {
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param amqpCacoon - an instance of AMQP Cacoon which will manage all AMQP communications.
     * @param amqpQueue - a string with the name of the queue to consume from.
     * @param logger - a logger instance to use for logging.
     * @param options - options for the behavior of the provider.
     **/
    constructor(amqpCacoon, amqpQueue, logger, options) {
        this.alreadyRegistered = false;
        // Save the constructor parameters to local class variables
        this.logger = logger;
        this.amqpCacoon = amqpCacoon;
        this.amqpQueue = amqpQueue;
        this.options = options;
    }
    /**
     * registerHandler
     *
     * Does this by...
     * 1. Points the passed in applyInputCb to a class instance handler
     * 2. If this is the first call then we register an amqpCacoon consumer which in turn
     *    registers an amqpHandler function that will receive all new AMQP messages.
     *
     * @param applyInputCb - a handler that will be called when there is input. It should be passed input and context.
     * @returns Promise<void>
     **/
    registerInput(applyInputCb) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputAmqp.registerHandler: Start`);
            // Link applyInputCb to a class property we can reference later
            this.handler = applyInputCb;
            if (!this.alreadyRegistered) {
                // Register the local handler with the amqp provider
                yield this.amqpCacoon.registerConsumer(this.amqpQueue, this.amqpHandler.bind(this));
                // Make so we only register one consumer
                this.alreadyRegistered = true;
            }
            (_b = this.logger) === null || _b === void 0 ? void 0 : _b.trace(`CoreInputAmqp.registerHandler: End`);
        });
    }
    /**
     * amqpHandler
     *
     * This is the function that we register with the amqpCacoon.
     * This function loops through the handlers registered and passes
     * the amqp message to our local registered handlers as a message.
     *
     * Note tha this INPUT provider adds a SPECIFIC object into facts with property amqpMessage.
     * amqpMessage in turn has three objects:
     * - amqpMessageContent: a string containing the AMQP message content.
     * - amqpMessageFields: an object, the amqp fields object.
     * - amqpMessageProperties: an object, the amqp properties object.
     *
     * @param channel: the active/open AMQP channel (inside a ChannelWrapper from node-amqp-connection-manager) to consume from.
     * @param msg: object - the message object, ConsumeMessage type.
     * @return Promise<void>
     **/
    amqpHandler(channel, msg) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputAmqp.amqpHandler: Start`);
            try {
                // Create an object for our message - note we DO NOT validate the message here at all!
                // It will be set to a string with whatever. it's the responsibility of the application
                // using this Input to validate and THROW an error with the name 'MessageValidationError'
                // which this method will catch and treat differently!
                let amqpMessage = {
                    amqpMessageContent: msg.content.toString(),
                    amqpMessageFields: msg.fields,
                    amqpMessageProperties: msg.properties,
                };
                // And an object for our context
                let context = {};
                // Call the inputContextCallback if we were passed one
                if (this.options.inputContextCallback) {
                    context = lodash_1.default.merge(context, this.options.inputContextCallback(msg));
                }
                (_b = this.logger) === null || _b === void 0 ? void 0 : _b.debug(`CoreInputAmqp.amqpHandler - amqpMessage: ${util_1.default.inspect(amqpMessage)}`);
                // Call our handler with an object for facts called amqpMessage that wraps our
                // ICoreAmqpMessage. Why? This input can't "own" the facts structure of the rules engine
                // since that should be driven by an application. Therefore, we just put in 1 item into
                // facts, an amqpMessage and then let the application do whatever it wants/needs.
                // Note, the handler WILL return updated facts, but we're not using them in this input!
                const resultFacts = yield this.handler({ amqpMessage: amqpMessage }, context);
                // Ack the message
                channel.ack(msg);
            }
            catch (e) {
                // Handle errors!
                // If this is a message validation error then we do not need to retry because this is not going to magically be fixed.
                // Note: Applications using this Input are expected to return an error name == 'MessageValidationError'
                // to trigger this first condition! These messages, since they are invalid, are ACK so that they don't
                // requeue forever! (They're expected to fail any retry, since they're invalid!)
                if (e.name === 'MessageValidationError') {
                    (_c = this.logger) === null || _c === void 0 ? void 0 : _c.error(`CoreInputAmqp.amqpHandler: Validation Error: ${e.message}`);
                    (_d = this.logger) === null || _d === void 0 ? void 0 : _d.trace(`The message contained: ${msg.content.toString()}`);
                    (_e = this.logger) === null || _e === void 0 ? void 0 : _e.trace(`The message fields contained: ${util_1.default.inspect(msg.fields)}`);
                    (_f = this.logger) === null || _f === void 0 ? void 0 : _f.trace(`The message properties contained: ${util_1.default.inspect(msg.properties)}`);
                    // Since we're dealing with INVALID messages (as defined by the application using this Input) we
                    // ACK so we don't requeue the messages forever!
                    channel.ack(msg);
                }
                else {
                    // Otherwise, the error is something other than INVALID MESSAGE, so we deal with it.
                    (_g = this.logger) === null || _g === void 0 ? void 0 : _g.error(`CoreInputAmqp.amqpHandler - Will Nack Message - INNER ERROR: ${e.message}`);
                    // TODO: Implement a requeueHandler callback - it will get the ORIGINAL message and context since
                    //  we don't have access to the changed facts, and will control requeue more than just "same queue". It
                    //  should work in concert with requeueOnNack.
                    // Nack the message and requeue only based on the options passed to this input
                    channel.nack(msg, false, this.options.requeueOnNack || false);
                }
            }
            (_h = this.logger) === null || _h === void 0 ? void 0 : _h.trace(`CoreInputAmqp.amqpHandler: End`);
        });
    }
}
exports.default = CoreInputAmqp;
//# sourceMappingURL=amqp-input.js.map