"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Load dependencies
 */
// @ts-ignore
const later_1 = __importDefault(require("@breejs/later"));
// This is a simple abstraction to help manage repeated tasks
class TaskScheduler {
    /**
     * Constructor for the TaskScheduler
     * @param {logger, schedulerConfig} - a logger so this class can log
     * @constructor
     */
    constructor({ logger, schedulerConfig, scheduledFn, }) {
        var _a;
        this.logger = logger;
        this.schedulerConfig = schedulerConfig;
        this.scheduledFn = scheduledFn;
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug('TaskScheduler -> constructor');
        switch (schedulerConfig.timezoneConfig) {
            case 'local':
                later_1.default.date.localTime();
                break;
            case 'utc':
                later_1.default.date.UTC();
                break;
        }
        // Initialize task list
        this.taskList = [];
        this.isStarted = false;
    }
    /**
     * start - This must be called before the tasks actually start running
     *
     * @params none
     * return undefined
     **/
    start() {
        var _a, _b;
        // Start the scheduler
        if (!this.isStarted) {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug('TaskScheduler -> start - Starting all tasks');
            // Add configured tasks to to our internal task list
            for (let idx in this.schedulerConfig.tasks) {
                this._addToInternalTaskList(this.schedulerConfig.tasks[idx]);
            }
            // Schedule the actual tasks
            for (let idx in this.taskList) {
                this._scheduleTask(this.taskList[idx]);
            }
            this.isStarted = true;
        }
        else {
            (_b = this.logger) === null || _b === void 0 ? void 0 : _b.debug('TaskScheduler -> start - Tasks already started');
        }
    }
    /**
     * stop - This is called to stop the scheduler
     *
     * @params none
     * return undefined
     **/
    stop() {
        var _a, _b, _c, _d;
        // Start the scheduler
        if (this.isStarted) {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug('TaskScheduler -> start - Stopping all tasks');
            // Schedule the actual tasks
            for (let idx in this.taskList) {
                (_c = (_b = this.taskList[idx]) === null || _b === void 0 ? void 0 : _b.handle) === null || _c === void 0 ? void 0 : _c.clear();
            }
            this.isStarted = false;
        }
        else {
            (_d = this.logger) === null || _d === void 0 ? void 0 : _d.debug('TaskScheduler -> start - Tasks already stopped');
        }
    }
    /**
     * Schedule a task - This should be called from the start function and not externally
     * Does this by calling an interval timer using the later.js library
     *
     * @params task    - Object with the task config (name/schedule/taskFn/skipIfBackup)
     * return undefined
     **/
    _scheduleTask(task) {
        var _a;
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug('TaskScheduler -> _scheduleTask - Scheduling task: ', task.name);
        // Later js is a library for starting tasks at deterministic times (ie start of the hour, min or second)
        // Store the schedule handle just in-case we decide to stop tasks at some
        // later date (also useful for test cleanup)
        if (task.schedule) {
            task.handle = later_1.default.setInterval(() => {
                // Add task to the queue to make sure we only run 1 of the same task at a time
                this._addToTaskQueue(task);
            }, task.schedule);
        }
    }
    /**
     * Add to the task's promise queue
     *
     * @params task - Object with the task config (name/schedule/taskFn/skipIfBackup/promiseQueue)
     ***/
    _addToTaskQueue(task) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (!task.promiseQueue) {
                // NOTE: p-queue is a es module - To use esm within commonjs module, you have to do a dynamic import. Unfortunatly, dynamic
                // imports are converted to require in typescript. Eval is a work around to force this to use a dynamic import even within typescript
                const { default: PQueue } = yield eval('import("p-queue")');
                task.promiseQueue = new PQueue({ concurrency: task.concurrency });
            }
            // Add task to the promise queue
            if (task.promiseQueue) {
                if (task.promiseQueue.size > task.maxTaskQueueLength) {
                    (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error(`TaskScheduler -> Task Error: Queue length exeeeded maximum size of ${task.maxTaskQueueLength} for task `);
                    return;
                }
                try {
                    yield task.promiseQueue.add(() => __awaiter(this, void 0, void 0, function* () {
                        var _d;
                        // Just to log the start of the task
                        (_d = this.logger) === null || _d === void 0 ? void 0 : _d.info(`TaskScheduler -> Task Starting: ${task.name}`);
                        yield this.scheduledFn(task);
                    }));
                    // Log the task successfully completed
                    (_b = this.logger) === null || _b === void 0 ? void 0 : _b.info(`TaskScheduler -> Task Successfully run: ${task.name}`);
                }
                catch (err) {
                    // Log task error
                    (_c = this.logger) === null || _c === void 0 ? void 0 : _c.info(`TaskScheduler -> Task: ${task.name} - Error: `, err);
                }
            }
        });
    }
    /**
     * This function is intended for internal use only.
     * It takes a task config from conf/task-scheduler.json
     *
     * @params taskConfig    - Object with task name and schedule config
     *                         and file with class that has a static execute funciton
     * return undefined      - Or throws error if it fails
     ***/
    _addToInternalTaskList(taskConfig) {
        var _a, _b, _c;
        try {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug('TaskScheduler -> _addToInternalTaskList - Adding task: ', taskConfig.name);
            let taskConcurrency = taskConfig.concurrency ||
                this.schedulerConfig.defaultPerTaskConcurrency ||
                Infinity;
            // Create the later.js schedule
            let schedule;
            if (taskConfig.intervalText) {
                // Using the text parser
                schedule = later_1.default.parse.text(taskConfig.intervalText);
            }
            else if (taskConfig.intervalCron) {
                // Using the cron parser
                schedule = later_1.default.parse.cron(taskConfig.intervalCron);
            }
            else {
                throw new Error('Invalid schedule');
            }
            (_b = this.logger) === null || _b === void 0 ? void 0 : _b.debug('TaskScheduler -> _addToInternalTaskList - task: ', taskConfig.name, ' - interval: ', taskConfig.intervalText || taskConfig.intervalCron, ` - Next Schedules: ${later_1.default.schedule(schedule).next(1)}`);
            this.taskList.push({
                name: taskConfig.name || 'Unknown name',
                maxTaskQueueLength: taskConfig.maxTaskQueueLength ||
                    this.schedulerConfig.defaultMaxPerTaskQueueLength ||
                    Infinity,
                concurrency: taskConcurrency,
                taskConfig: taskConfig,
                input: taskConfig.input,
                schedule,
            });
        }
        catch (err) {
            (_c = this.logger) === null || _c === void 0 ? void 0 : _c.error(`TaskScheduler -> _addToInternalTaskList - Error:${err}`);
            throw err;
        }
    }
}
exports.default = TaskScheduler;
//# sourceMappingURL=task-scheduler.js.map