"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// Bring in package dependencies
const lodash_1 = __importDefault(require("lodash"));
const task_scheduler_1 = __importDefault(require("./helpers/task-scheduler"));
class CoreInputSchedulerProvider {
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param schedulerConfig - Schedule configuration for the task scheduler
     * @param logger - a logger instance to use for logging.
     * @param options
     **/
    constructor(schedulerConfig, logger, options) {
        this.alreadyRegistered = false;
        // Save the constructor parameters to local class variables
        this.logger = logger;
        this.options = options;
        this.scheduler = new task_scheduler_1.default({
            logger,
            schedulerConfig,
            scheduledFn: this.schedulerHandler.bind(this),
        });
    }
    /**
     * registerHandler
     * This is called by the rules engine when it starts. This function starts the task scheduler
     *
     * Do this by...
     * 1. Set the rules engine apply function for use later
     * 2. If we are not already setup then start the scheduler
     *
     * @param applyInputCb - a handler that will be called when there is input. It should be passed input and context.
     * @returns Promise<void>
     **/
    registerInput(applyInputCb) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`CoreInputScheduler.registerHandler: Start`);
            // 1. Set the rules engine apply function for use later
            this.applyInputCb = applyInputCb;
            if (!this.alreadyRegistered) {
                // 2. If we are not already setup then start the scheduler
                this.scheduler.start();
                this.alreadyRegistered = true;
            }
            (_b = this.logger) === null || _b === void 0 ? void 0 : _b.debug(`CoreInputScheduler.registerHandler: End`);
        });
    }
    /**
     * schedulerHandler
     * Handle the scheduler interval
     *
     * does this by
     * 1. Build input facts
     * 2. Optionaly call inputContextCallback if it is defined to build a context
     * 3. Pass input facts and context to the rules engine
     */
    schedulerHandler(task) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.debug(`CoreInputScheduler.schedulerHandler: Start`);
            try {
                //  1. Build input facts
                let input = {
                    name: task.name,
                    intervalText: task.taskConfig.intervalText || undefined,
                    intervalCron: task.taskConfig.intervalCron || undefined,
                    input: task.input,
                    queue: {
                        waiting: task.promiseQueue.size,
                        pending: task.promiseQueue.pending,
                    },
                };
                // And an object for our context
                let context = {};
                // 2. Optionaly call inputContextCallback if it is defined to build a context
                if (this.options.inputContextCallback) {
                    context = lodash_1.default.merge(context, this.options.inputContextCallback(input));
                }
                // 3. Pass input facts and context to the rules engine
                yield this.applyInputCb({ scheduledTask: input }, context);
            }
            catch (ex) {
                (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error('ERROR: CoreInputScheduler.schedulerHandler', ex);
            }
            (_c = this.logger) === null || _c === void 0 ? void 0 : _c.debug(`CoreInputScheduler.schedulerHandler: End`);
        });
    }
    unregisterInput() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                try {
                    this.scheduler.stop();
                    this.alreadyRegistered = false;
                    resolve();
                }
                catch (e) {
                    reject(e);
                }
            });
        });
    }
}
exports.default = CoreInputSchedulerProvider;
//# sourceMappingURL=scheduler-input.js.map