"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// Bring in package dependencies
const lodash_1 = __importDefault(require("lodash"));
const node_dgram_1 = __importDefault(require("node:dgram"));
class CoreInputUdpProvider {
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param udpPorts - an array of numbers, the ports to start HTTP servers on.
     * @param logger - a logger instance to use for logging.
     * @param options
     **/
    constructor(udpPorts, logger, options) {
        this.alreadyRegistered = false;
        // Save the constructor parameters to local class variables
        this.logger = logger;
        this.udpPorts = udpPorts;
        this.options = options;
    }
    /**
     * registerHandler
     * This is called by the rules engine when it starts. This function sets up the udp listener
     *
     * Do this by...
     * 1. Set the rules engine apply function for use later
     * 2. If we are not already setup then loop over every configured port
     * 3. for every configured port create udp socket
     * 4. for every configured port bind socket
     * 5. for every configured port setup listening event
     * 6. for every configured port setup message handler
     * 7. for every configured port save the socket for later removal if we ever unregisterInput
     *
     * @param applyInputCb - a handler that will be called when there is input. It should be passed input and context.
     * @returns Promise<void>
     **/
    registerInput(applyInputCb) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputUdp.registerHandler: Start`);
            // Setup a handler for the Udp request
            // 1. Set the rules engine apply function for use later
            this.applyInputCb = applyInputCb;
            // Wire in udp bridge to this handler, if we've not done so already!
            if (!this.alreadyRegistered) {
                this.udpServer = [];
                // 2. If we are not already setup then loop over every configured port
                for (let p of this.udpPorts) {
                    // Configure then Start up a new instance of the Udp Bridge (note, this wires in the handler!)
                    // 3. for every configured port create udp socket
                    let socket = node_dgram_1.default.createSocket('udp4', this.udpHandler);
                    (_b = this.logger) === null || _b === void 0 ? void 0 : _b.trace(`CoreInputUdp.registerHandler: Starting server on port ${p}`);
                    // 4. for every configured port bind socket
                    socket.bind(p, () => {
                        var _a;
                        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputUdp.registerHandler: Server bound port ${p}`);
                    });
                    // 5. for every configured port setup listening event
                    socket.on('listening', () => {
                        var _a;
                        let address = socket.address();
                        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputUdp.registerHandler: Server listening on ${address.family} ${address.address}:${address.port}`);
                    });
                    // 6. for every configured port setup message handler
                    socket.on('message', this.udpHandler.bind(this));
                    // 7. for every configured port save the socket for later removal if we ever unregisterInput
                    this.udpServer.push(socket);
                }
                // Keep track so that we only register one udp bridge
                this.alreadyRegistered = true;
            }
            (_c = this.logger) === null || _c === void 0 ? void 0 : _c.trace(`CoreInputUdp.registerHandler: End`);
        });
    }
    /**
     * udpHandler
     * Handle the udp message
     *
     * does this by
     * 1. Build input facts
     * 2. Optionaly call inputContextCallback if it is defined to build a context
     * 3. Pass input facts and context to the rules engine
     */
    udpHandler(msg, rinfo) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreInputUdp.udpHandler: Start`);
            try {
                //  1. Build input facts
                let input = {
                    body: msg.toString(),
                    remoteInfo: rinfo,
                };
                // And an object for our context
                let context = {};
                // 2. Optionaly call inputContextCallback if it is defined to build a context
                if (this.options.inputContextCallback) {
                    context = lodash_1.default.merge(context, this.options.inputContextCallback(input));
                }
                // 3. Pass input facts and context to the rules engine
                yield this.applyInputCb({ udpRequest: input }, context);
            }
            catch (ex) {
                (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error('ERROR: CoreInputUdp.udpHandler', ex);
            }
            (_c = this.logger) === null || _c === void 0 ? void 0 : _c.trace(`CoreInputUdp.udpHandler: End`);
        });
    }
    unregisterInput() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                try {
                    for (let socket of this.udpServer) {
                        socket.close();
                    }
                    this.udpServer = [];
                    this.alreadyRegistered = false;
                    resolve();
                }
                catch (e) {
                    reject(e);
                }
            });
        });
    }
}
exports.default = CoreInputUdpProvider;
//# sourceMappingURL=udp-input.js.map