import { ILogger, IOutputProvider, IOutputResult } from '../../types';
import AmqpCacoon from 'amqp-cacoon';
/**
 * Rule Output Provider Amqp
 *
 * This class wires up the AmqpCacoon to a rule output provider for an AMQP Response.
 *
 * Usage:
 * 1. Instantiate the class and pass in the instantiated amqpCacoon and logger
 * 2. call registerHandler to register the input callback
 * 3. When input comes in, the handler registered in step 2 will be called
 *
 **/
export default class CoreOutputAmqp implements IOutputProvider {
    private alreadyRegistered;
    private logger?;
    private amqpCacoon;
    private amqpPublishChannelWrapper;
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param amqpCacoon - an instance of AMQP Cacoon which will manage all AMQP communications.
     * @param logger - a logger instance to use for logging.
     **/
    constructor(amqpCacoon: AmqpCacoon, logger: ILogger);
    /**
     * outputResult
     *
     * Outputs a result from a rules engine pass IF the result facts contains an AMQP Response!
     *
     * Does this by...
     * 1. Receives a result
     * 2. Analyzes the incoming result object <IOutputResult>result.
     * 3. If there is NO incoming error, pulls facts, inspects for an amqpPublishAction matching <ICoreAmqpPublishAction>.
     * 4. If it finds an <ICoreAmqpPublishAction>amqpPublishAction object, attempts to publish based on that.
     *
     * <ICoreAmqpPublishAction>
     * A rules engine implementation that wants to use the Core AMQP Output should pass the special object
     * amqpPublishAction as part of the facts object. That object in turn should contain the following, either as
     * a single instance OR as an ARRAY of these:
     * - amqpMessageContent: string - The content of the message, which should already be a string. Applications should
     *   JSON.stringify before passing.
     * - amqpPublishRoutingKey?: string - Optional, a routing key to use when publishing the message into the Exchange.
     * - amqpPublishOptions?: object, - Optional but if passed should be formed per
     *   https://www.squaremobius.net/amqp.node/channel_api.html#channel_publish. This allows setting headers, expiration
     *   amd more.
     *
     * @param result - <IOutputResult> the result of a rules engine pass, including error, facts and other object,
     *  as changed by the rules engine.
     * @returns Promise<void>
     **/
    outputResult(result: IOutputResult): Promise<void>;
}
