"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
/**
 * Rule Output Provider Amqp
 *
 * This class wires up the AmqpCacoon to a rule output provider for an AMQP Response.
 *
 * Usage:
 * 1. Instantiate the class and pass in the instantiated amqpCacoon and logger
 * 2. call registerHandler to register the input callback
 * 3. When input comes in, the handler registered in step 2 will be called
 *
 **/
class CoreOutputAmqp {
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param amqpCacoon - an instance of AMQP Cacoon which will manage all AMQP communications.
     * @param logger - a logger instance to use for logging.
     **/
    constructor(amqpCacoon, logger) {
        this.alreadyRegistered = false;
        // Save the constructor parameters to local class variables
        this.logger = logger;
        this.amqpCacoon = amqpCacoon;
    }
    /**
     * outputResult
     *
     * Outputs a result from a rules engine pass IF the result facts contains an AMQP Response!
     *
     * Does this by...
     * 1. Receives a result
     * 2. Analyzes the incoming result object <IOutputResult>result.
     * 3. If there is NO incoming error, pulls facts, inspects for an amqpPublishAction matching <ICoreAmqpPublishAction>.
     * 4. If it finds an <ICoreAmqpPublishAction>amqpPublishAction object, attempts to publish based on that.
     *
     * <ICoreAmqpPublishAction>
     * A rules engine implementation that wants to use the Core AMQP Output should pass the special object
     * amqpPublishAction as part of the facts object. That object in turn should contain the following, either as
     * a single instance OR as an ARRAY of these:
     * - amqpMessageContent: string - The content of the message, which should already be a string. Applications should
     *   JSON.stringify before passing.
     * - amqpPublishRoutingKey?: string - Optional, a routing key to use when publishing the message into the Exchange.
     * - amqpPublishOptions?: object, - Optional but if passed should be formed per
     *   https://www.squaremobius.net/amqp.node/channel_api.html#channel_publish. This allows setting headers, expiration
     *   amd more.
     *
     * @param result - <IOutputResult> the result of a rules engine pass, including error, facts and other object,
     *  as changed by the rules engine.
     * @returns Promise<void>
     **/
    outputResult(result) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreOutputAmqp.outputResult: Start`);
            // Open our publish channel, but only if we've not already done it!
            if (!this.alreadyRegistered) {
                // Open our Publish Channel
                this.amqpPublishChannelWrapper =
                    yield this.amqpCacoon.getPublishChannel();
                // Make so we only register one publish channel
                this.alreadyRegistered = true;
            }
            // Process the incoming results
            try {
                if (result.error) {
                    // The rules engine already sent in an error, so we just log and not output
                    (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error(`CoreOutputAmqp.outputResult: the result object contained an error. Nothing will publish. The error received was: ${util_1.default.inspect(result.error)}`);
                }
                else if ((_c = result === null || result === void 0 ? void 0 : result.facts) === null || _c === void 0 ? void 0 : _c.amqpPublishAction) {
                    // No error + we have an facts.amqpPublishAction so we publish!
                    // Setup an array of actions so we can handle multiple.
                    let amqpPublishArray = [];
                    // Were we passed an ARRAY of amqpPublishAction in facts.amqpPublishAction?
                    if (Array.isArray(result.facts.amqpPublishAction)) {
                        // It is an array, so we just set our local to the array we were passed
                        amqpPublishArray = result.facts.amqpPublishAction;
                    }
                    else {
                        // Not an array, so let's add the one action into our local array
                        amqpPublishArray.push(result.facts.amqpPublishAction);
                    }
                    // Now, let's loop over the array and publish each
                    for (let amqpPublishAction of amqpPublishArray) {
                        const amqpPublishExchange = amqpPublishAction.amqpPublishExchange;
                        const amqpPublishRoutingKey = amqpPublishAction.amqpPublishRoutingKey || '';
                        const amqpMessageContent = amqpPublishAction.amqpMessageContent || '';
                        const amqpPublishOptions = amqpPublishAction.amqpPublishOptions;
                        // Publish!
                        yield this.amqpPublishChannelWrapper.publish(amqpPublishExchange, amqpPublishRoutingKey, Buffer.from(amqpMessageContent), amqpPublishOptions);
                        // Log success
                        (_d = this.logger) === null || _d === void 0 ? void 0 : _d.info(`CoreOutputAmqp.outputResult: Message published to exchange='${amqpPublishExchange}' with routingKey='${amqpPublishRoutingKey}'.`);
                    }
                }
                else {
                    // We don't have an amqpPublishAction, so we log that.
                    (_e = this.logger) === null || _e === void 0 ? void 0 : _e.error(`CoreOutputAmqp.outputResult: Error in retrieving an amqpPublishAction from result.facts. Nothing was published.`);
                }
            }
            catch (e) {
                // Oh no! Something else. Log the error.
                (_f = this.logger) === null || _f === void 0 ? void 0 : _f.error(`CoreOutputAmqp.outputResult: Error - INNER ERROR: ${e.message}`);
            }
            (_g = this.logger) === null || _g === void 0 ? void 0 : _g.trace(`CoreOutputAmqp.outputResult: End`);
        });
    }
}
exports.default = CoreOutputAmqp;
//# sourceMappingURL=amqp-output.js.map