import { ILogger, IOutputProvider, IOutputResult } from '../../types';
import AmqpCacoon from 'amqp-cacoon';
/**
 * Rule Output Provider Amqp RPC (remote procedure call)
 *
 * This class wires up the AmqpCacoon to a rule output provider for an AMQP RPC Response.
 *
 * Usage:
 * 1. Instantiate the class and pass in the instantiated amqpCacoon and logger
 * 2. call registerHandler to register the input callback
 * 3. When input comes in, the handler registered in step 2 will be called
 *
 **/
export default class CoreOutputAmqpRpc implements IOutputProvider {
    private alreadyRegistered;
    private logger?;
    private amqpCacoon;
    private amqpPublishChannelWrapper;
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param amqpCacoon - an instance of AMQP Cacoon which will manage all AMQP communications.
     * @param logger - a logger instance to use for logging.
     **/
    constructor(amqpCacoon: AmqpCacoon, logger: ILogger);
    /**
     * outputResult
     *
     * Outputs a result from a rules engine pass IF the result facts contains an AMQP RPC Response!
     *
     * Does this by...
     * 1. Receives a result
     * 2. Analyzes the incoming result object <IOutputResult>result.
     * 3. If there is NO incoming error, pulls facts, inspects for an amqpRpcPublishAction matching <ICoreAmqpRpcPublishAction>.
     * 4. If it finds an <ICoreAmqpRpcPublishAction>amqpRpcPublishAction object, attempts to publish based on that.
     *
     * <ICoreAmqpRpcPublishAction>
     * A rules engine implementation that wants to use the Core AMQP RPC Output should pass the special object
     * amqpRpcPublishAction as part of the facts object. That object in turn should contain the following, as
     * a single instance (array not supported here and if passed, only the first item will be returned):
     * - amqpMessageContent: string - The content of the message, which should already be a string. Applications should
     *   JSON.stringify before passing.
     *
     * Note that the replyTo queue and correlationId (both part of the AMQP RPC protocol) are extracted from the
     * original input message properties. Also, per the RPC Protocol spec, the REPLY TO QUEUE must exist already
     * since it should be created/asserted by the client application that sends the original message.
     *
     * @param result - <IOutputResult> the result of a rules engine pass, including error, facts and other object,
     *  as changed by the rules engine.
     * @returns Promise<void>
     **/
    outputResult(result: IOutputResult): Promise<void>;
}
