"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
/**
 * Rule Output Provider Amqp RPC (remote procedure call)
 *
 * This class wires up the AmqpCacoon to a rule output provider for an AMQP RPC Response.
 *
 * Usage:
 * 1. Instantiate the class and pass in the instantiated amqpCacoon and logger
 * 2. call registerHandler to register the input callback
 * 3. When input comes in, the handler registered in step 2 will be called
 *
 **/
class CoreOutputAmqpRpc {
    /**
     * constructor
     *
     * This function sets class level variables.
     *
     * @param amqpCacoon - an instance of AMQP Cacoon which will manage all AMQP communications.
     * @param logger - a logger instance to use for logging.
     **/
    constructor(amqpCacoon, logger) {
        this.alreadyRegistered = false;
        // Save the constructor parameters to local class variables
        this.logger = logger;
        this.amqpCacoon = amqpCacoon;
    }
    /**
     * outputResult
     *
     * Outputs a result from a rules engine pass IF the result facts contains an AMQP RPC Response!
     *
     * Does this by...
     * 1. Receives a result
     * 2. Analyzes the incoming result object <IOutputResult>result.
     * 3. If there is NO incoming error, pulls facts, inspects for an amqpRpcPublishAction matching <ICoreAmqpRpcPublishAction>.
     * 4. If it finds an <ICoreAmqpRpcPublishAction>amqpRpcPublishAction object, attempts to publish based on that.
     *
     * <ICoreAmqpRpcPublishAction>
     * A rules engine implementation that wants to use the Core AMQP RPC Output should pass the special object
     * amqpRpcPublishAction as part of the facts object. That object in turn should contain the following, as
     * a single instance (array not supported here and if passed, only the first item will be returned):
     * - amqpMessageContent: string - The content of the message, which should already be a string. Applications should
     *   JSON.stringify before passing.
     *
     * Note that the replyTo queue and correlationId (both part of the AMQP RPC protocol) are extracted from the
     * original input message properties. Also, per the RPC Protocol spec, the REPLY TO QUEUE must exist already
     * since it should be created/asserted by the client application that sends the original message.
     *
     * @param result - <IOutputResult> the result of a rules engine pass, including error, facts and other object,
     *  as changed by the rules engine.
     * @returns Promise<void>
     **/
    outputResult(result) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        return __awaiter(this, void 0, void 0, function* () {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.trace(`CoreOutputAmqpRpc.outputResult: Start`);
            // Open our publish channel, but only if we've not already done it!
            if (!this.alreadyRegistered) {
                // Open our Publish Channel
                this.amqpPublishChannelWrapper =
                    yield this.amqpCacoon.getPublishChannel();
                // Make so we only register one publish channel
                this.alreadyRegistered = true;
            }
            // Process the incoming results
            try {
                if (result.error) {
                    // The rules engine already sent in an error, so we just log and not output
                    (_b = this.logger) === null || _b === void 0 ? void 0 : _b.error(`CoreOutputAmqpRpc.outputResult: the result object contained an error. Nothing will publish. The error received was: ${util_1.default.inspect(result.error)}`);
                }
                else if (((_c = result === null || result === void 0 ? void 0 : result.facts) === null || _c === void 0 ? void 0 : _c.amqpMessage) &&
                    ((_d = result === null || result === void 0 ? void 0 : result.facts) === null || _d === void 0 ? void 0 : _d.amqpRpcPublishAction)) {
                    // No error + we have an original amqpMessage + amqpRpcPublishAction so we publish!
                    const originalMessage = result.facts.amqpMessage;
                    const replyTo = originalMessage.amqpMessageProperties.replyTo;
                    const correlationId = originalMessage.amqpMessageProperties.correlationId;
                    // Do we have a replyTo and a correlationId?
                    if (replyTo && correlationId) {
                        // Setup an array of actions so we can handle multiple (though not supported
                        // we check just in case).
                        let amqpRpcPublishArray = [];
                        // Were we passed an ARRAY of amqpPublishAction in facts.amqpPublishAction?
                        if (Array.isArray(result.facts.amqpRpcPublishAction)) {
                            // It is an array, so we just set our local to the array we were passed
                            amqpRpcPublishArray = result.facts.amqpRpcPublishAction;
                        }
                        else {
                            // Not an array, so let's add the one action into our local array
                            amqpRpcPublishArray.push(result.facts.amqpRpcPublishAction);
                        }
                        // Now, we force a response of a single item, the first one!
                        const amqpRpcResponse = amqpRpcPublishArray[0].amqpRpcResponse || '';
                        // Publish!
                        yield this.amqpPublishChannelWrapper.sendToQueue(replyTo, Buffer.from(amqpRpcResponse), { correlationId: correlationId });
                        // Log success
                        (_e = this.logger) === null || _e === void 0 ? void 0 : _e.info(`CoreOutputAmqpRpc.outputResult: Message published to reply-to='${replyTo}'.`);
                    }
                    else {
                        // We don't have a replyTo nor correlationId, so we log that.
                        (_f = this.logger) === null || _f === void 0 ? void 0 : _f.error(`CoreOutputAmqpRpc.outputResult: Error in executing amqpRpcPublishAction from result.facts. The original message did not have the required properties "reply_to" and "correlation_id". Unable to respond via AMQP RPC.`);
                    }
                }
                else {
                    if (!((_g = result === null || result === void 0 ? void 0 : result.facts) === null || _g === void 0 ? void 0 : _g.amqpMessage) &&
                        ((_h = result === null || result === void 0 ? void 0 : result.facts) === null || _h === void 0 ? void 0 : _h.amqpRpcPublishAction)) {
                        // We have an amqpRpcPublishAction, but we can't retrieve the original amqpMessage.
                        // This means we won't be able to respond since the original message is needed to know how to respond!
                        (_j = this.logger) === null || _j === void 0 ? void 0 : _j.error(`CoreOutputAmqpRpc.outputResult: Error in retrieving the original amqpMessage for an amqpRpcPublishAction from result.facts. Nothing was published.`);
                    }
                }
            }
            catch (e) {
                // Oh no! Something else. Log the error.
                (_k = this.logger) === null || _k === void 0 ? void 0 : _k.error(`CoreOutputAmqpRpc.outputResult: Error - INNER ERROR: ${e.message}`);
            }
            (_l = this.logger) === null || _l === void 0 ? void 0 : _l.trace(`CoreOutputAmqpRpc.outputResult: End`);
        });
    }
}
exports.default = CoreOutputAmqpRpc;
//# sourceMappingURL=amqp-rpc-output.js.map